-- vim: ts=2 sw=2 ai et enc=utf8 fenc=utf8
RaidChecklist = {}

local locale = GetLocale ()

-- This table contains the localized pet families. These MUST BE localized, or the pets won't get mapped to their de(buffs).
-- There's no way to get hold of an un-localized pet family name it seems :-/
RaidChecklistPetFamilies = {}
local P = RaidChecklistPetFamilies

-- Master string table, ideally localized.
RaidChecklistStrings = {}
local L = RaidChecklistStrings

-- We load the enUS strings as defaults, so if a locale doesn't override it, the enUS version gets used
--@localization(locale="enUS", format="lua_additive_table", table-name="P", namespace="PetFamilies")@
--@localization(locale="enUS", format="lua_additive_table", table-name="L", namespace="", handle-subnamespaces="none")@

if locale == "enUS" then
  P.SupportedLocale = true
else
  -- Assume unsupported until told otherwise
  P.SupportedLocale = nil

  -- Buffs & debuff names, default to the official Blizzard categories until everyone's got them added
  -- Apparently Blizz was "helpful" and line-wrapped & hyphenated these in some languages. /sigh
  L.CRIT            = RAID_BUFF_7
  L.ATTACKPWR       = RAID_BUFF_3
  L.SPELLPWR        = RAID_BUFF_5
  L.PHYSHASTE       = RAID_BUFF_4
  L.SPELLHASTE      = RAID_BUFF_6
  L.STATS           = RAID_BUFF_1
  L.STAM            = RAID_BUFF_2
  L.MASTERY         = RAID_BUFF_8
end

if locale == "deDE" then
--@localization(locale="deDE", format="lua_additive_table", table-name="P", namespace="PetFamilies")@
--@localization(locale="deDE", format="lua_additive_table", table-name="L", namespace="", handle-subnamespaces="none")@
elseif locale == "esES" then
--@localization(locale="esES", format="lua_additive_table", table-name="P", namespace="PetFamilies")@
--@localization(locale="esES", format="lua_additive_table", table-name="L", namespace="", handle-subnamespaces="none")@
elseif locale == "esMX" then
--@localization(locale="esMX", format="lua_additive_table", table-name="P", namespace="PetFamilies")@
--@localization(locale="esMX", format="lua_additive_table", table-name="L", namespace="", handle-subnamespaces="none")@
elseif locale == "frFR" then
--@localization(locale="frFR", format="lua_additive_table", table-name="P", namespace="PetFamilies")@
--@localization(locale="frFR", format="lua_additive_table", table-name="L", namespace="", handle-subnamespaces="none")@
elseif locale == "itIT" then
--@localization(locale="itIT", format="lua_additive_table", table-name="P", namespace="PetFamilies")@
--@localization(locale="itIT", format="lua_additive_table", table-name="L", namespace="", handle-subnamespaces="none")@
elseif locale == "koKR" then
--@localization(locale="koKR", format="lua_additive_table", table-name="P", namespace="PetFamilies")@
--@localization(locale="koKR", format="lua_additive_table", table-name="L", namespace="", handle-subnamespaces="none")@
elseif locale == "ptBR" then
--@localization(locale="ptBR", format="lua_additive_table", table-name="P", namespace="PetFamilies")@
--@localization(locale="ptBR", format="lua_additive_table", table-name="L", namespace="", handle-subnamespaces="none")@
elseif locale == "ruRU" then
--@localization(locale="ruRU", format="lua_additive_table", table-name="P", namespace="PetFamilies")@
--@localization(locale="ruRU", format="lua_additive_table", table-name="L", namespace="", handle-subnamespaces="none")@
elseif locale == "zhCN" then
--@localization(locale="zhCN", format="lua_additive_table", table-name="P", namespace="PetFamilies")@
--@localization(locale="zhCN", format="lua_additive_table", table-name="L", namespace="", handle-subnamespaces="none")@
elseif locale == "zhTW" then
--@localization(locale="zhTW", format="lua_additive_table", table-name="P", namespace="PetFamilies")@
--@localization(locale="zhTW", format="lua_additive_table", table-name="L", namespace="", handle-subnamespaces="none")@
end

-- Spec id -> localized name lookup
local spec2name = {}
for i=1,500 do
  id, name = GetSpecializationInfoByID(i)
  if id then
    spec2name[id] = name
  end
end

function RCSpecName (gid)
  local name = spec2name[gid]
  if name then
    name = "("..name..")"
  end
  return name
end


-- Colorized class names
local classes = {}
FillLocalizedClassList (classes)
L.HUNTER      = ClassColorString ("HUNTER")     .. classes.HUNTER
L.WARRIOR     = ClassColorString ("WARRIOR")    .. classes.WARRIOR
L.DEATHKNIGHT = ClassColorString ("DEATHKNIGHT").. classes.DEATHKNIGHT
L.PRIEST      = ClassColorString ("PRIEST")     .. classes.PRIEST
L.SHAMAN      = ClassColorString ("SHAMAN")     .. classes.SHAMAN
L.PALADIN     = ClassColorString ("PALADIN")    .. classes.PALADIN
L.DRUID       = ClassColorString ("DRUID")      .. classes.DRUID
L.WARLOCK     = ClassColorString ("WARLOCK")    .. classes.WARLOCK
L.ROGUE       = ClassColorString ("ROGUE")      .. classes.ROGUE
L.MAGE        = ClassColorString ("MAGE")       .. classes.MAGE
L.MONK        = ClassColorString ("MONK")       .. classes.MONK
L[UNKNOWN]    = ClassColorString (UNKNOWN)      .. UNKNOWN

-- Specializations, now automatically looked up - yay!
L.AFFLICTION    = RCSpecName (265)
L.ARCANE        = RCSpecName ( 62)
L.ARMS          = RCSpecName ( 71)
L.ASSASSINATION = RCSpecName (259)
L.BALANCE       = RCSpecName (102)
L.BEASTMASTER   = RCSpecName (253)
L.BLOOD         = RCSpecName (250)
L.BREWMASTER    = RCSpecName (268)
L.COMBAT        = RCSpecName (260)
L.DEMONOLOGY    = RCSpecName (266)
L.DESTRUCTION   = RCSpecName (267)
L.DISCIPLINE    = RCSpecName (256)
L.ELEMENTAL     = RCSpecName (262)
L.ENHANCEMENT   = RCSpecName (263)
L.FERAL         = RCSpecName (103)
L.FIRE          = RCSpecName ( 63)
L.FROSTMAGE     = RCSpecName ( 64)
L.FROSTDK       = RCSpecName (251)
L.FURY          = RCSpecName ( 72)
L.GUARDIAN      = RCSpecName (104)
L.HOLYPALADIN   = RCSpecName ( 65)
L.HOLYPRIEST    = RCSpecName (257)
L.MARKSMAN      = RCSpecName (254)
L.MISTWEAVER    = RCSpecName (270)
L.PROTPALADIN   = RCSpecName ( 66)
L.PROTWARRIOR   = RCSpecName ( 73)
L.RESTOSHAMAN   = RCSpecName (264)
L.RESTODRUID    = RCSpecName (105)
L.RETRIBUTION   = RCSpecName ( 70)
L.SHADOW        = RCSpecName (258)
L.SUBTLETY      = RCSpecName (261)
L.SURVIVAL      = RCSpecName (255)
L.UNHOLY        = RCSpecName (252)
L.WINDWALKER    = RCSpecName (269)
-- Ferocity = 74
-- Cunning = 79
-- Tenacity = 81

-- Cardinality
L[0]              = "" -- Don't show a number for missing abilities
L[1]              = "" -- If there's only one, we don't really need to show a number for it (in fact it looks silly when soloing)
L[2]              = "(2)"
L[3]              = "(3)"
L[4]              = "(4)"
L[5]              = "(5)"
L[6]              = "(6)"
L[7]              = "(7)"
L[8]              = "(8)"
L[9]              = "(9)"
L.MANY            = "(*)" -- Life is easier if we can keep things the same width (single digits). Of course, this might be a very western notion... feel free to educate me!

-- Flag whether we should even attempt to run, given the state of the localization data
if not P.SupportedLocale then
  print ("|cffff0000>> Raid Checklist is not supported for your locale ("..GetLocale()..") yet, sorry")
  RaidChecklist = nil
  return
end

